local _G = getfenv(0)
local xo, yo = 8, 3 -- X-offset, Y-offset
local equiped = {} -- Table to store equiped items
local socketsTable = {
	-- /dump string.split(":", GetInventoryItemLink("player", i))
	-- Prismatic Sockets in 7.0, how many are there?
	[1808] = true, -- From Heroic Dungeons and T19 Normal Raids
	[3386] = true, -- From Vendor
	-- 3rd Socket opened in the Artifact Weapon
	--[724] = true, -- Claws of Ursoc (Guardian Druid)
	--[750] = true, -- Strom'kar, the Warbreaker (Arms Warrior)
}
local f = CreateFrame("Frame", nil, _G.PaperDollFrame) -- iLvel number frame
f:RegisterEvent("PLAYER_LOGIN")

-- Tooltip and scanning by Phanx @ http://www.wowinterface.com/forums/showthread.php?p=271406
local S_ITEM_LEVEL = "^" .. gsub(ITEM_LEVEL, "%%d", "(%%d+)")
local S_UPGRADE_LEVEL = "^" .. gsub(ITEM_UPGRADE_TOOLTIP_FORMAT, "%%d", "(%%d+)")

local scantip = CreateFrame("GameTooltip", "iLvlScanningTooltip", nil, "GameTooltipTemplate")
scantip:SetOwner(UIParent, "ANCHOR_NONE")

local function _getRealItemLevel(slotId)
	local realItemLevel, currentUpgradeLevel, maxUpgradeLevel
	local hasItem = scantip:SetInventoryItem("player", slotId)
	if not hasItem then return nil end -- With this we don't get ilvl for offhand if we equip 2h weapon

	for i = 2, scantip:NumLines() do -- Line 1 is always the name so you can skip it.
		local text = _G["iLvlScanningTooltipTextLeft"..i]:GetText()
		if text and text ~= "" then
			realItemLevel = realItemLevel or strmatch(text, S_ITEM_LEVEL)
			if not (currentUpgradeLevel or maxUpgradeLevel) then
				currentUpgradeLevel, maxUpgradeLevel = strmatch(text, S_UPGRADE_LEVEL)
			end

			if realItemLevel and currentUpgradeLevel and maxUpgradeLevel then
				return realItemLevel, tonumber(currentUpgradeLevel), tonumber(maxUpgradeLevel)
			end
		end
	end

	return realItemLevel
end

local function _updateItems()
	for i = 1, 17 do -- Only check changed items or items without ilvl text, skip the shirt (4)
		local itemLink = GetInventoryItemLink("player", i)
		if itemLink and i ~= 4 and (equiped[i] ~= itemLink or f[i]:GetText() == nil) then
			equiped[i] = itemLink
			local upgradeString, enchantString, gemString = "", "", ""
			local realItemLevel, currentUpgradeLevel, maxUpgradeLevel = _getRealItemLevel(i)
			local _, _, enchantID, gem1, gem2, gem3, gem4, _, _, _, _, _, _, numBonuses, affixes = strsplit(":", itemLink, 15)
			enchantID = tonumber(enchantID); numBonuses = tonumber(numBonuses); realItemLevel = realItemLevel or ""

			if currentUpgradeLevel and maxUpgradeLevel and currentUpgradeLevel < maxUpgradeLevel then
				upgradeString = "|TInterface\\PetBattles\\BattleBar-AbilityBadge-Strong:0:0:0:0:32:32:2:30:2:30|t"
			end
			if i == 2 or i == 3 or i == 11 or i == 12 or i == 15 then
				-- Neck, Shoulders, Finger0, Finger1, Chest
				if enchantID and enchantID > 0 then
					enchantString = "|T136244:0:0:0:0:32:32:2:30:2:30|t"
				else
					enchantString = "|T136244:0:0:0:0:32:32:2:30:2:30:221:0:0|t"
				end
			end
			for _, v in pairs({ gem1, gem2, gem3, gem4 }) do
				local _, _, _, _, _, _, _, _, _, t = GetItemInfo(v)
				if t and t > 0 then
					gemString = gemString.."|T"..t..":0:0:0:0:32:32:2:30:2:30|t"
				end
			end
			if numBonuses and numBonuses > 0 then
				for b = 1, numBonuses do
					local bonusID = select(b, strsplit(":", affixes))
					if socketsTable[tonumber(bonusID)] then
						--gemString = gemString.."|TInterface\\ItemSocketingFrame\\UI-EmptySocket-Prismatic:0:0:0:0:32:32:2:30:2:30:221:0:0|t"
						gemString = gemString.."|TInterface\\ItemSocketingFrame\\UI-EmptySocket-Red:0:0:0:0:32:32:2:30:2:30|t"
					end
				end
			end

			if i <= 5 or i == 15 or i == 9 then -- Left side
				f[i]:SetText(string.format("%s %s%s%s", realItemLevel, enchantString, gemString, upgradeString))
			elseif i == 16 then -- MainHand
				f[i]:SetText(string.format("%s %s\n%s", upgradeString, realItemLevel, gemString))
			elseif i == 17 then -- OffHand
				f[i]:SetText(string.format("%s %s\n%s", realItemLevel, upgradeString, gemString))
			else -- Right side
				f[i]:SetText(string.format("%s%s%s %s", upgradeString, gemString, enchantString, realItemLevel))
			end

			--[[if currentUpgradeLevel and maxUpgradeLevel and currentUpgradeLevel < maxUpgradeLevel then
				if i <= 5 or i == 15 or i == 9 or i == 17 then -- Left side
					f[i]:SetText(realItemLevel.."|TInterface\\PetBattles\\BattleBar-AbilityBadge-Strong:0:0:0:0:32:32:2:30:2:30|t")
				else -- Right side
					f[i]:SetText("|TInterface\\PetBattles\\BattleBar-AbilityBadge-Strong:0:0:0:0:32:32:2:30:2:30|t"..realItemLevel)
				end
			else
				f[i]:SetText(realItemLevel)
			end]]
			--f[i]:SetText(_getRealItemLevel(i))
		end
	end
end

local function _createStrings()
	local function _stringFactory(parent, myPoint, parentPoint, x, y)
		local s = f:CreateFontString(nil, "OVERLAY", "GameFontNormalOutline")
		s:SetPoint(myPoint, parent, parentPoint, x or 0, y or 0)

		return s
	end

	f:SetFrameLevel(_G.CharacterHeadSlot:GetFrameLevel())

	f[1] = _stringFactory(_G.CharacterHeadSlot, "LEFT", "RIGHT", xo)
	f[2] = _stringFactory(_G.CharacterNeckSlot, "LEFT", "RIGHT", xo)
	f[3] = _stringFactory(_G.CharacterShoulderSlot, "LEFT", "RIGHT", xo)
	f[15] = _stringFactory(_G.CharacterBackSlot, "LEFT", "RIGHT", xo)
	f[5] = _stringFactory(_G.CharacterChestSlot, "LEFT", "RIGHT", xo)
	f[9] = _stringFactory(_G.CharacterWristSlot, "LEFT", "RIGHT", xo)

	f[10] = _stringFactory(_G.CharacterHandsSlot, "RIGHT", "LEFT", -xo)
	f[6] = _stringFactory(_G.CharacterWaistSlot, "RIGHT", "LEFT", -xo)
	f[7] = _stringFactory(_G.CharacterLegsSlot, "RIGHT", "LEFT", -xo)
	f[8] = _stringFactory(_G.CharacterFeetSlot, "RIGHT", "LEFT", -xo)
	f[11] = _stringFactory(_G.CharacterFinger0Slot, "RIGHT", "LEFT", -xo)
	f[12] = _stringFactory(_G.CharacterFinger1Slot, "RIGHT", "LEFT", -xo)
	f[13] = _stringFactory(_G.CharacterTrinket0Slot, "RIGHT", "LEFT", -xo)
	f[14] = _stringFactory(_G.CharacterTrinket1Slot, "RIGHT", "LEFT", -xo)

	f[16] = _stringFactory(_G.CharacterMainHandSlot, "BOTTOM", "TOP", 0, yo)
	f[17] = _stringFactory(_G.CharacterSecondaryHandSlot, "BOTTOM", "TOP", 0, yo)

	f:Hide()
end

local function OnEvent(self, event, ...) -- Event handler
	if event == "PLAYER_LOGIN" then
		self:UnregisterEvent(event)

		_createStrings()

		_G.PaperDollFrame:HookScript("OnShow", function(self)
			f:RegisterEvent("PLAYER_EQUIPMENT_CHANGED")
			f:RegisterEvent("ITEM_UPGRADE_MASTER_UPDATE")
			_updateItems()
			f:Show()
		end)

		_G.PaperDollFrame:HookScript("OnHide", function(self)
			f:UnregisterEvent("PLAYER_EQUIPMENT_CHANGED")
			f:UnregisterEvent("ITEM_UPGRADE_MASTER_UPDATE")
			f:Hide()
		end)
	elseif event == "PLAYER_EQUIPMENT_CHANGED" or event == "ITEM_UPGRADE_MASTER_UPDATE" then
		_updateItems()
	end
end
f:SetScript("OnEvent", OnEvent)